package com.qs.mp.web.controller.api.admin;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qs.mp.admin.domain.Coupon;
import com.qs.mp.admin.domain.CouponTicket;
import com.qs.mp.admin.domain.param.CouponParam;
import com.qs.mp.admin.domain.param.GoodsParam;
import com.qs.mp.admin.domain.vo.CouponVO;
import com.qs.mp.admin.service.ICouponService;
import com.qs.mp.admin.service.ICouponTicketService;
import com.qs.mp.common.constant.UserConstants;
import com.qs.mp.common.core.domain.AjaxResult;
import com.qs.mp.common.core.page.TableDataInfo;
import com.qs.mp.common.enums.CouponStatusEnum;
import com.qs.mp.common.enums.ErrorCodeEnum;
import com.qs.mp.system.domain.SysUser;
import com.qs.mp.system.service.ISysUserService;
import com.qs.mp.web.controller.common.BaseApiController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import ma.glasnost.orika.MapperFacade;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @auther liugl
 * @create 2022-03-09 23:45:48
 * @describe 代金券管理前端控制器
 */
@Api("代金券管理API")
@RestController
@RequestMapping("/api/v1/mp/admin/coupon/*")
@Component
public class CouponMgrController extends BaseApiController {

	@Autowired
	private ICouponService couponService;

	@Autowired
	private ICouponTicketService couponTicketService;

	@Autowired
	private ISysUserService userService;

	@Autowired
	private MapperFacade mapperFacade;

	/**
	 * 获取我的下级代金券列表信息，支持翻页
	 *
	 * @return
	 */
	@PostMapping("/list")
	public TableDataInfo listCoupon(@RequestBody Coupon coupon) {
		List<Coupon> list = new ArrayList<Coupon>();
		startPage();
		LambdaQueryWrapper<Coupon> queryWrapper = new LambdaQueryWrapper<Coupon>();
		queryWrapper.like(null != coupon && StringUtils.isNotBlank(coupon.getTitle()), Coupon::getTitle, coupon.getTitle());
		queryWrapper.like(null != coupon && null != coupon.getStatus(), Coupon::getStatus, coupon.getStatus());
		queryWrapper.orderByAsc(Coupon::getCreatedTime);
		list = couponService.list(queryWrapper);
		return getDataTable(list);
	}


	/**
	 * 新增代金券信息
	 * @param
	 * @return
	 */
	@ApiOperation(value = "新增代金券信息", notes = "代金券端新增代金券")
	@PostMapping("/create")
	public AjaxResult couponCreate(@Validated @RequestBody CouponParam couponParam) {
		if (StringUtils.isNotBlank(couponParam.getCouponId())) {
			return AjaxResult.error("该代金券已存在");
		}
		Coupon coupon = mapperFacade.map(couponParam, Coupon.class);
		// 1、校验名称是否重复（代金券表）
		LambdaQueryWrapper<Coupon> queryWrapper = new LambdaQueryWrapper<>();
		queryWrapper.eq(Coupon::getTitle, coupon.getTitle());
		int nameCount = couponService.count(queryWrapper);
		if(nameCount > 0) {
			return AjaxResult.error("代金券名称" + coupon.getTitle() + "已存在！");
		}
		// 使用范围
		List<CouponTicket> ticketList = new ArrayList<>();
		if(StringUtils.isNotBlank(couponParam.getBoxIds())) {
			String[] boxIdArray = couponParam.getBoxIds().split(",");
	        for (int i = 0; i < boxIdArray.length; i++) {
	        	String boxId = boxIdArray[i];
	        	CouponTicket couponTicket = new CouponTicket();
	        	couponTicket.setBoxId(boxId);
	        }
		}
		// 3.插入数据
		try {
			couponService.saveCoupon(coupon,ticketList);
		} catch (Exception e) {
			return AjaxResult.error("代金券'" + coupon.getTitle() + "'新增失败" + e.getMessage());
		}

		return AjaxResult.success("代金券'" + coupon.getTitle() + "'新增成功");
	}

	/**
	 * 编辑代金券信息
	 * @param
	 * @return
	 */
	@ApiOperation(value = "编辑代金券信息", notes = "代金券端编辑代金券")
	@PostMapping("/update")
	public AjaxResult couponUpdate(@Validated @RequestBody CouponParam couponParam) {
		if (null == couponParam || StringUtils.isBlank(couponParam.getCouponId())) {
			return error(ErrorCodeEnum.ERROR_CODE_1001);
		}
		Coupon coupon = mapperFacade.map(couponParam, Coupon.class);
		// 1、校验修改代金券是否为当前用户的代金券
		Coupon oldCoupon = couponService.getById(coupon.getCouponId());
		if(null == oldCoupon || null == oldCoupon.getCouponId()) {
			return AjaxResult.error("代金券'" + oldCoupon.getTitle() + "'编辑失败，代金券ID异常");
		}
		// 2.校验名称是否重复（代金券表）；
		if(!coupon.getTitle().equals(oldCoupon.getTitle())) {
			LambdaQueryWrapper<Coupon> queryWrapper = new LambdaQueryWrapper<>();
			queryWrapper.eq(Coupon::getTitle, coupon.getTitle());
			int nameCount = couponService.count(queryWrapper);
			if(nameCount > 0) {
				return AjaxResult.error("代金券名称" + coupon.getTitle() + "已存在！");
			}
		}
		// 使用范围
		List<CouponTicket> ticketList = new ArrayList<>();
		if(StringUtils.isNotBlank(couponParam.getBoxIds())) {
			String[] boxIdArray = couponParam.getBoxIds().split(",");
	        for (int i = 0; i < boxIdArray.length; i++) {
	        	String boxId = boxIdArray[i];
	        	CouponTicket couponTicket = new CouponTicket();
	        	couponTicket.setBoxId(boxId);
	        	couponTicket.setCouponId(coupon.getCouponId());
	        }
		}
		try {
			couponService.updateCoupon(coupon, ticketList);
		} catch (Exception e) {
			return AjaxResult.error(e.getMessage());
		}
		return AjaxResult.success("代金券'" + coupon.getTitle() + "'编辑成功");
	}


	/**
	 * 停用、启用代金券
	 * @param
	 * @return
	 */
	@ApiOperation(value = "停用、启用代金券信息", notes = "代金券管理编辑代金券")
	@PostMapping("/status")
	public AjaxResult couponStatus(@RequestBody CouponParam couponParam) {
		String couponId = (null != couponParam && StringUtils.isNotBlank(couponParam.getCouponId()))?couponParam.getCouponId():"";
		CouponStatusEnum status = null != couponParam ?couponParam.getStatus():null;
		if (StringUtils.isBlank(couponId)
				|| null == status) {
			return error(ErrorCodeEnum.ERROR_CODE_1001);
		}
		try {
			couponService.lambdaUpdate().set(Coupon::getStatus, couponParam.getStatus()).eq(Coupon::getCouponId, couponId).update();
			// 查询代金券信息
		} catch (Exception e) {
			return AjaxResult.error("操作失败");
		}
		return AjaxResult.success("操作成功");
	}

}
